// Copyright (C) 2010 - DIGITEO - Michael Baudin

// Updates the .xml files by deleting existing files and 
// creating them again from the .sci with help_from_sci.

function b = isContentUptodate ( content , filename )
  // Check if a content matches a file.
  // 
  // Calling Sequence
  //   b = isContentUptodate ( content , filename )
  //
  // Parameters
  //   content : a 1x1 matrix of strings, the new content
  //   filename : a 1x1 matrix of strings, the file to be checked.
  //   b : a 1x1 matrix of booleans
  //
  // Description
  //   Returns true if the <literal>filename</literal> is up-to-date, that is,
  //   if the file does not require to be changed.
  //   The file is to be updated if one of the following conditions is satisfied.
  // <itemizedlist>
  //   <listitem>
  //     The file does not exist.
  //   </listitem>
  //   <listitem>
  //     The file exists, but its content is not equal to <literal>content</literal>.
  //     The content comparison ignores the leading and trailing blanks.
  //   </listitem>
  // </itemizedlist>
  //   Last update : 01/10/2010.
  //
  // Example
  //   // TODO...
  //
  // Author
  //   2010 - DIGITEO - Michael Baudin
  //   
  
  //
  [lhs, rhs] = argn()
  apifun_checkrhs ( "isContentUptodate" , rhs , 2 )
  apifun_checklhs ( "isContentUptodate" , lhs , 1 )
  //
  apifun_checktype ( "isContentUptodate" , content , "content" , 1 , "string" )
  apifun_checktype ( "isContentUptodate" , filename , "filename" , 2 , "string" )
  //
  apifun_checkveccol ( "isContentUptodate" , content , "content" , 1 , size(content,"*") )
  apifun_checkscalar ( "isContentUptodate" , filename , "filename" , 2 )
  //
  b = %t
  if ( fileinfo(filename) == [] ) then
    b = %f
  else
    txt = mgetl(filename)
    if ( or ( stripblanks(content) <> stripblanks(txt) ) ) then
      b = %f
    end
  end
endfunction

function b = fileUpdateIfNeeded ( content , filename )
  // Update a file if this is needed.
  //
  // Calling Sequence
  //   b = fileUpdateIfNeeded ( content , filename )
  //
  // Parameters
  //   content : a matrix of strings, the new content
  //   filename : a 1x1 matrix of strings, the file to be checked.
  //   b : a 1x1 matrix of booleans
  //
  // Description
  //   Returns true if the <literal>filename</literal> is up-to-date, that is,
  //   if the file was not changed.
  //   Returns false if the file was changed.
  //   The file is to be updated according to the rules of <literal>isContentUptodate</literal>.
  //   Generates an error if the file was to be changed, but that was not possible.
  //   Last update : 02/09/2010.
  //
  // Example
  //   // TODO...
  //
  // Author
  //   2010 - DIGITEO - Michael Baudin
  //   

  [lhs, rhs] = argn()
  apifun_checkrhs ( "fileUpdateIfNeeded" , rhs , 2 )
  apifun_checklhs ( "fileUpdateIfNeeded" , lhs , 1 )
  //
  apifun_checktype ( "fileUpdateIfNeeded" , content , "content" , 1 , "string" )
  apifun_checktype ( "fileUpdateIfNeeded" , filename , "filename" , 2 , "string" )
  //
  apifun_checkveccol ( "fileUpdateIfNeeded" , content , "content" , 1 , size(content,"*") )
  apifun_checkscalar ( "fileUpdateIfNeeded" , filename , "filename" , 2 )
  //

  b = isContentUptodate ( content , filename )
  if ( ~b ) then
    r = mputl ( content , filename )
    if ( ~r ) then
      error(sprintf(gettext("%s: Unable to write xml file: %s\n"),"fileUpdateIfNeeded",filename));
    end
  end
endfunction

function helpupdate ( funarray , helpdir , macrosdir , demosdir , modulename )
  // Update the help and the demos from the .sci files.
  //
  // Calling Sequence
  //   updatehelp ( funarray , helpdir , macrosdir , demosdir )
  //
  // Parameters
  //   funarray : column matrix of strings, the list of functions to update
  //   helpdir : a 1x1 matrix of strings, the help directory
  //   macrosdir : a 1x1 matrix of strings, the macros directory
  //   demosdir : a 1x1 matrix of strings, the demonstration directory
  //   modulename : a 1x1 matrix of strings, the name of the module
  //
  // Description
  //   Update the .xml help files and the demos scripts
  //   from the macros corresponding to the function array
  //   of strings funarray.
  //   The existing .xml files in the help dir which 
  //   correspond to file in the funarray are deleted (Caution !).
  //   Generates the .xml and the .sce files from the help_from_sci function.
  //   Generates a demonstration gateway.
  //
  //   If demosdir is an empty matrix, do not generate the demonstrations.
  //   Last update : 02/09/2010.
  //
  // Author
  //   2010, Michael Baudin

  //
  [lhs, rhs] = argn()
  apifun_checkrhs ( "helpupdate" , rhs , 5 )
  apifun_checklhs ( "helpupdate" , lhs , 1 )

  //
  apifun_checktype ( "helpupdate" , funarray , "funarray" , 1 , "string" )
  apifun_checktype ( "helpupdate" , helpdir , "helpdir" , 2 , "string" )
  apifun_checktype ( "helpupdate" , macrosdir , "macrosdir" , 3 , "string" )
  apifun_checktype ( "helpupdate" , demosdir , "demosdir" , 4 , "string" )
  apifun_checktype ( "helpupdate" , modulename , "modulename" , 5 , "string" )
  //
  apifun_checkveccol ( "helpupdate" , funarray , "funarray" , 1 , size(funarray,"*") )
  apifun_checkscalar ( "helpupdate" , helpdir , "helpdir" , 2 )
  apifun_checkscalar ( "helpupdate" , macrosdir , "macrosdir" , 3 )
  apifun_checkscalar ( "helpupdate" , demosdir , "demosdir" , 4 )
  apifun_checkscalar ( "helpupdate" , modulename , "modulename" , 5 )
  //
  if ( fileinfo ( helpdir ) == [] ) then
    error(sprintf(gettext("%s: Wrong help directory: %s does not exist.\n"),"updatehelp",filenormalize(helpdir)));
  end
  if ( fileinfo ( macrosdir ) == [] ) then
    error(sprintf(gettext("%s: Wrong macros directory: %s does not exist.\n"),"updatehelp",filenormalize(macrosdir)));
  end
  if ( demosdir <> [] ) then
    if ( fileinfo ( demosdir ) == [] ) then
      error(sprintf(gettext("%s: Wrong demos directory: %s does not exist.\n"),"updatehelp",filenormalize(demosdir)));
    end
  end
  
  //
  // Make funarray into a vector column
  funarray = funarray(:)
  //
  // 2. Generate each .xml and each .sce from the .sci
  flist = ls(macrosdir)';
  for f = flist
    issci = regexp(f,"/(.*).sci/");
    kf = find(funarray==basename(f))
    if ( issci <> [] & kf <> [] ) then
      scifile = fullfile ( macrosdir , f )
      mprintf("Processing %s\n",scifile);
      funname = funarray(kf)
      xmlfile = fullfile ( helpdir , funname + ".xml" )
      // Generate the xml and the demo content
      [helptxt,demotxt]= help_from_sci (scifile)
      // Delete the "info" tag, containing the date (3 lines)
      k = find(stripblanks(helptxt)=="<info>")
      helptxt(k:k+2) = []
      // Update the xml file, if necessary
      isuptodate = fileUpdateIfNeeded ( helptxt , xmlfile )
      if ( isuptodate ) then
        changetxt = "(XML Up-to-date)"
      else
        changetxt = "XML Changed"
      end
      mprintf("  %s %s\n",changetxt,xmlfile);
      // Create the demo
      if ( demosdir <> [] ) then
        demofile = fullfile ( demosdir , funname + ".sce" )
        // Update the demo script
        header = []
        header($+1) = "//"
        header($+1) = "// This help file was automatically generated from "+funname+".sci using help_from_sci()."
        header($+1) = "// PLEASE DO NOT EDIT"
        header($+1) = "//"
        footer = []
        footer($+1) = msprintf("//\n");
        footer($+1) = msprintf("// Load this script into the editor\n");
        footer($+1) = msprintf("//\n");
        footer($+1) = msprintf("filename = ""%s"";\n",funname + ".sce");
        footer($+1) = msprintf("dname = get_absolute_file_path(filename);\n");
        footer($+1) = msprintf("editor ( fullfile(dname,filename) );\n");
        demotxt = [header;demotxt;footer]
        // Update the demo file, if necessary
        isuptodate = fileUpdateIfNeeded ( demotxt , demofile )
        if ( isuptodate ) then
          changetxt = "(SCE Up-to-date)"
        else
          changetxt = "SCE Changed"
        end
        mprintf("  %s %s\n",changetxt,demofile);
      end
    end
  end
  //
  // 3. Generates the Demonstration gateway
  // Include all .sce files in the gateway, including 
  // handcrafted scripts which might have been written
  // by the user (and not generated by this function).
  // To make so, use the list of .sce files in the demo dir and 
  /// not simply the funarray.
  if ( demosdir <> [] ) then
    gatetxt = []
    gatetxt ($+1) = "// This help file was automatically generated using helpupdate"
    gatetxt ($+1) = "// PLEASE DO NOT EDIT"
    gatetxt ($+1) = "demopath = get_absolute_file_path(""" + modulename + ".dem.gateway.sce"");"
    gatetxt ($+1) = "subdemolist = ["
    flist = ls(demosdir)';
    for f = flist
      issce = regexp(f,"/(.*).sce/");
      isgateway = regexp(f,"/(.*).gateway.sce/");
      if ( issce <> [] & isgateway == [] ) then
        flen = length(f)
        funname = part(f,[1:flen-4])
        gatetxt($+1) = """" + funname + """, """ + funname + ".sce""; .."
      end
    end
    //
    gatetxt ($+1) = "];"
    gatetxt ($+1) = "subdemolist(:,2) = demopath + subdemolist(:,2)"
    gatefile = fullfile ( demosdir , modulename+".dem.gateway.sce" )
    // Update the gateway file, if necessary
    isuptodate = fileUpdateIfNeeded ( gatetxt , gatefile )
    if ( isuptodate ) then
      changetxt = "(SCE Gateway Up-to-date)"
    else
      changetxt = "SCE Gateway Changed"
    end
    mprintf("%s %s\n",changetxt,gatefile);
  end
endfunction

function str = filenormalize(name)
  // Normalize a path to a file or directory.
  //
  // Calling Sequence
  //   str = filenormalize(name)
  //
  // Parameters
  //   name : a 1x1 matrix of strings, the path to a file, directory or link
  //   str : a 1x1 matrix of strings, the normalized path 
  //
  // Description
  //   Replaces \ with /
  //  For a complete implementation, see :
  //  http://flibs.cvs.sourceforge.net/viewvc/flibs/src/filedir/m_vfile.f90?revision=1.4&view=markup
  //
  // Examples
  // name = "F:\ForgeScilab\assert\help\en_US";
  // str = filenormalize(name)
  // expected = "F:/ForgeScilab/assert/help/en_US"
  //
  
  str = strsubst(name,"\","/")
endfunction

//
cwd = get_absolute_file_path("update_help.sce");
mprintf("Working dir = %s\n",cwd);
//
// Generate the library help
mprintf("Updating number\n");
helpdir = cwd;
funmat = [
  "number_carmichael"
  "number_coprime"
  "number_extendedeuclid"
  "number_factor"
  "number_fermat"
  "number_gcd"
  "number_inversemod"
  "number_isdivisor"
  "number_isprime"
  "number_lcm"
  "number_mersenne"
  "number_powermod"
  "number_primes"
  "number_primes1000"
  "number_primorial"
  "number_probableprime"
  "number_pseudoprime"
  "number_solvelinmod"
  ];
macrosdir = cwd +"../../macros";
demosdir = cwd +"../../demos";
modulename = "number";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename )

//
// Generate the library help
mprintf("Updating support\n");
helpdir = cwd+ "support";
funmat = [
  "number_cget"
  "number_check"
  "number_checkrange"
  "number_configure"
  "number_startup"
  ];
macrosdir = cwd +"../../macros";
demosdir = cwd +"../../demos";
modulename = "number";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename )

//
// Generate the library help
mprintf("Updating conversion\n");
helpdir = cwd+ "conversion";
funmat = [
  "number_barygui"
  "number_baryguiclose"
  "number_frombary"
  "number_tobary"
  ];
macrosdir = cwd +"../../macros";
demosdir = cwd +"../../demos";
modulename = "number";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename )

