// Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin

// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// number_extendedeuclid --
//   
function [ d , x , y ] = number_extendedeuclid ( a , b )
  // Solves a linear Diophantine equation (Bezout identity).
  //
  // Calling Sequence
  //   d = number_extendedeuclid ( a , b )
  //   [ d , x ] = number_extendedeuclid ( a , b )
  //   [ d , x , y ] = number_extendedeuclid ( a , b )
  //
  // Parameters
  //   a : a 1x1 matrix of floating point integers, must be positive
  //   b : a 1x1 matrix of floating point integers, must be positive
  //   d : a 1x1 matrix of floating point integers
  //   x : a 1x1 matrix of floating point integers
  //   y : a 1x1 matrix of floating point integers
  //
  // Description
  //   Returns d, x and y such that d is the GCD of a and b and
  //   d = x*a + y*b.
  //
  //   Uses the extended Euclid's algorithm.
  //
  // Examples
  // [ d , x , y ] = number_extendedeuclid ( 99 , 78 ) // [3 -11 14]
  // [ d , x , y ] = number_extendedeuclid ( 120 , 23 ) // [1 -9 47]
  //
  // Bibliography
  //   "Introduction to algorithms", Cormen, Leiserson, Rivest, Stein, 2nd edition
  //
  // Authors
  // Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
  //

  number_check ( a )
  number_checkpositive ( "a" , a )
  number_check ( b )
  number_checkpositive ( "b" , b )
  if ( b == 0 ) then
    d = a
    x = 1
    y = 0
    return
  end
  [ dp , xp , yp ] = number_extendedeuclid ( b , pmodulo ( a , b ) )
  d = dp
  x = yp
  y = xp - floor ( a / b ) * yp
endfunction

