// Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
// Copyright (C) INRIA - Farid BELAHCENE
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function varargout = number_factor ( varargin )
  // Factors a number.
  //
  // Calling Sequence
  //   result = number_factor ( n )
  //   [ result , status ] = number_factor ( n )
  //   [ result , status ] = number_factor ( n , method )
  //
  // Parameters
  //  n : a 1x1 matrix of floating point integers, must be positive
  //  method : a 1x1 matrix of strings, the method to use (default "fasttrialdivision"). 
  //  result : a row matrix containing the divisors
  //  status : %t if the factorization is a sucecss, %f if the factorization is a failure
  //
  // Description
  //   Returns the prime factors of n, as a row matrix.
  //   If n=0, 1, 2 or 3, then returns result=n and status = %t.
  //   
  //  In case of success, all factors are returned in the result row matrix,
  //  and success = %t.
  //
  //  In case of failure, success = %f and the factor which were found
  //  are returned in result.
  //  In this case, result = n if no factor has been found.
  //
  // The list of available methods is the following.
  //      <variablelist>
  //        <varlistentry>
  //          <term> method="trialdivision" </term>
  //          <listitem>
  //          <para>
  //          Uses the Trial Division method, which is known as the
  //          simplest, and the least efficient method.
  //          This is Algorithm A, "Factoring by division", section 4.5.4 in Knuth's TAO.
  //          </para>
  //          </listitem>
  //        </varlistentry>
  //        <varlistentry>
  //          <term> method="fasttrialdivision" </term>
  //          <listitem>
  //          <para>
  //          Uses a vectorized trial division algorithm.
  //          Uses an array with size <literal>sqrt(n)</literal>,
  //          which implies that
  //          it may fail for large integers.
  //          This implementation is due to F. Belahcene.
  //          </para>
  //          </listitem>
  //        </varlistentry>
  //        <varlistentry>
  //          <term> method="memorylesstd" </term>
  //          <listitem>
  //          <para>
  //          Uses the Trial Division method, without any list of primes.
  //          This is Algorithm A, "Factoring by division", section 4.5.4 in Knuth's TAO.
  //          We divide by prime until the remainder is not zero.
  //          The following primes are tested in order:
  //          2 3 5 7 11 13 17 ...
  //          After the prime p=5 has been used, we add 2, then 4, then 2, ...
  //          </para>
  //          </listitem>
  //        </varlistentry>
  //        <varlistentry>
  //          <term> method="fermat" </term>
  //          <listitem>
  //          <para>
  //          Uses the Fermat's method, without sieve.
  //          </para>
  //          </listitem>
  //          </varlistentry>
  //          <varlistentry>
  //          <term> method="pollard-rho" </term>
  //          <listitem>
  //          <para>
  //          Uses the Pollard-rho method with the collection of offsets
  //          associated with the "-pollardoffsets" option.
  //          The <literal>c</literal> integers in the "-pollardoffsets"
  //          are all used ( in order ) in the pseudo-random number generator
  //          <literal>x^2-c</literal>.
  //          </para>
  //          </listitem>
  //        </varlistentry>
  //      </variablelist>
  //
  //  With Pollard's rho method, failure is obtained either when the
  //  number is prime or composite (i.e. failure is not a mark 
  //  for primality or compositeness, it is just a failure).
  //  In that case, running the algorithm again, one or more time,
  //  may produce additionnal factors.
  //
  // Examples
  // number_factor ( 2^2-1 ) // 3
  // number_factor ( 2^3-1 ) // 7 
  // number_factor ( 2^4-1 ) // [3 5] 
  // number_factor ( 2^5-1 ) // 31 
  //
  // // Factor Mersenne numbers
  // algos = [
  //   "fasttrialdivision"
  //  "trialdivision"
  //  "memorylesstd"
  //  "fermat"
  //  "pollard-rho"
  // ];
  // for n = [2^7-1 2^8-1 2^9-1 2^11-1 2^12-1 2^13-1 2^14-1 2^15-1]
  //   mprintf("n=%d :\n",n);
  //   for method = algos'
  //     f = number_factor(n,method);
  //     mprintf("  %20s : %s\n",method,sci2exp(f));
  //   end
  // end
  //
  // Bibliography
  // "The Art of Computer Programming", Volume 2, Seminumerical Algorithms, Donald Knuth, Addison-Wesley
  //
  // Authors
  // Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
  // Copyright (C) INRIA - Farid BELAHCENE
  //

  [lhs,rhs]=argn();
  if ( ( rhs <> 1 ) & ( rhs <> 2 ) ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while %d or %d are expected."), "number_factor" , rhs , 1 , 2 );
    error(errmsg)
  end
  if ( ( lhs <> 1 ) & ( lhs <> 2 ) ) then
    errmsg = msprintf(gettext("%s: Unexpected number of output arguments : %d provided while %d or %d are expected.\n"), "number_factor" , lhs , 1 , 2 );
    error(errmsg)
  end
  select rhs
  case 1 then
    n = varargin ( 1 )
    method = "fasttrialdivision";
  case 2 then
    n = varargin ( 1 )
    method = varargin ( 2 )
  end
  number_check ( n )
  number_checkpositive ( "n" , n )
  if ( or(n==[0 1 2 3]) ) then 
    select lhs
    case 1 then
      varargout ( 1 ) = n
    case 2 then
      varargout ( 1 ) = n
      varargout ( 2 ) = %t
    end
    return
  end
  imax = number_cget ( "-imax" )
  select method
  case "trialdivision"
    [ result , status ] = factor_trialdivision ( n , imax )
  case "fasttrialdivision"
    [ result , status ] = factor_fasttd ( n )
  case "memorylesstd"
    [ result , status ] = factor_memorylesstd ( n , imax )
  case "fermat"
    [ result , status ] = factor_fermat ( n , imax )
  case "pollard-rho"
    [ result , status ] = factor_pollard ( n , imax )
  else
    errmsg = sprintf ( gettext ( "%s: Unknown method %s" ) , "number_factor" , method )
    error(errmsg)
  end
  if ( ~status & ( lhs == 1 ) ) then
    mprintf(gettext("%s: Warning! The algorithm failed. Try to increase the value of the -imax option.\n"), "number_isprime" );
  end
  select lhs
  case 1 then
    varargout ( 1 ) = result
  case 2 then
    varargout ( 1 ) = result
    varargout ( 2 ) = status
  end
endfunction
// 
// factor_fasttd --
//   Uses a vectorized trial division algorithm.
//   Uses an array with size sqrt(n).
// Author : F.Belahcene
//
function [ result , status ] = factor_fasttd ( x )
  s = floor ( sqrt ( n ) )
  number_log ( sprintf ( "Computing primes less than %s\n" , string ( s ) ) )
  xprimefact = number_primes ( s )
  result = []
  status = %f
  for i = 1 : imax
    number_log ( sprintf ( "Loop #%d/%d, x=%s\n" , i , imax , string ( x ) ) )
    if ( x==1 ) then
      number_log ( sprintf ( "Found x=1, done.\n" ) )
      status = %t
      break
    end
    xdiv = x ./ xprimefact
    xprimefact_ind = find(xdiv-floor(xdiv)==0)
    if ( xprimefact_ind <> [] )
      xprimefact = xprimefact ( xprimefact_ind )
      number_log ( sprintf ( "Found factors %s \n" , strcat ( string ( xprimefact ) , " " ) ) )
      result = [xprimefact result]
      x = x/prod(xprimefact)
    else
      number_log ( sprintf ( "Found no divisor, done.\n" ) )
      result=[x result]
      status = %t
      break
    end
  end
  if ( ~status ) then
    number_log ( sprintf ( "Failed to factor completely, x = %s.\n" , string ( x ) ) )
    result ( 1 , $ + 1 ) = x
  end
  result = gsort(result,'c','i')
endfunction

//
// factorization_trialdivision --
//   Returns the list of prime factors of n
//   Uses the Trial Division method, which is known as the 
//   simplest, and the least efficient method.
// Arguments:
//   n : the positive integer number to factor
//   imax : the maximum number of iterations in the search for a divisor
// Notes:
//   This is Algorithm A, "Factoring by division", section 4.5.4 in Knuth's TAO
//
function [ result , status ] = factor_trialdivision ( n , imax )
    result = []
    // Get all primes up to sqrt(n) + 1
    s = floor ( sqrt ( n ) ) + 1
    number_log ( sprintf ( "Computing primes less than %s\n" , string ( s ) ) )
    prms = number_primes ( s )
    if ( prms( $ ) < s ) then
      // Find a prime greater or equal to s
      p = prms( $ )
      while ( %t )
        p = p + 2
        [ isprime , status ] = number_isprime ( p )
        if ( ~status ) then
          status = %f
          break
        end
        if ( isprime ) then
          prms ( $ + 1 ) = p
          break
        end
      end
    end
    // Divide by prime until the remainder is not zero
    u = n
    // ip is the index of the current prime
    ip = 1
    p = prms ( ip )
    nbp = size ( prms , "*" )
    status = %f
    for i = 1 : imax
      number_log ( sprintf ( "Loop #%d/%d, u=%s, trying p = %s\n" , i , imax , string ( u ) , string ( p ) ) )
      if ( u == 1 ) then
        number_log ( sprintf ( "Found u=1, done.\n" ) )
        status = %t
        break
      end
      r = modulo ( u , p )
      if ( r==0 ) then
        number_log ( sprintf ( "Found divisor p=%s.\n" , string ( p ) ) )
        result ( 1 , $ + 1 ) = p
        u = u / p
      else
        // p does not divide u
        if ( ip < nbp ) then
          // There are primes left to test
          ip = ip + 1
          p = prms ( ip )
          number_log ( sprintf ( "Trying new divisor #%d, p=%s.\n" , ip , string ( p ) ) )
        else
          // There are no primes left to test : the last number n is a prime
          number_log ( sprintf ( "No prime left to test, done.\n" ) )
          result ( 1 , $ + 1 ) = u
          status = %t
          break
        end
      end
    end
    if ( u == 1 ) then
      number_log ( sprintf ( "Found u=1, done.\n" ) )
      status = %t
      return
    end
    if ( ~status ) then
      number_log ( sprintf ( "Failed to factor completely, u = %s.\n" , string ( u ) ) )
      result ( 1 , $ + 1 ) = u
    end
endfunction
//
// factor_fermat --
//   Returns the list of prime factors of n
//   Uses the Fermat's method.
//   The first number in the list is allways 1.
// Arguments:
//   n : the positive integer number to factor
//   imax : the maximum number of iterations in the search for a divisor
//
function [ result , status ] = factor_fermat ( n , imax )
  // Factors by two until n is odd
  [ twofactors , status ] = factor_bytwo ( n , imax )
  if ( ~status ) then
    result = []
    return
  end
  result = twofactors ( 1 : $ - 1 )
  u = twofactors ( $ )
  if ( u==1 ) then
    return
  end
  // Compute factors for odd number u
  [ otherfactors , status ] = factor_fermatdo ( u , imax )
  result = [ result otherfactors ]
  result = gsort ( result , "g" , "i" )
endfunction
//
// factor_fermatdo --
//   Performs the algorithm recursively on the 
//   given odd number.
//   Returns only the factors which are different from 1.
//   This is Algorithm C, "Factoring by addition and substraction", 
//   section 4.5.4 in Knuth's TAO
// Arguments:
//   n : the positive, odd, integer number to factor 
//   imax : the maximum number of iterations in the search for a divisor
// Note :
// The algorithm is very slow to factor 33554431.
// factor_fermatdo ( 33554431 )
//
function [ result , status ] = factor_fermatdo ( n , imax )
    [ p1 , p2 , status ] = factor_fermatcore ( n , imax )
    if ( ~status ) then
      result = n
      return
    end
    isprime = ( ( p1==1 ) & ( p2==n ) )
    // Factor p1 and p2 recursively, if required
    result = []
    if ( isprime ) then
      // If n is prime, there is no need for recursive calls
      // since n = 1 x n (where the 1 factor is discarded)
      result ( 1 , $ + 1 ) = p2
    else
      // Factorize p1
      [ otherfactors1 , status ] = factor_fermatdo ( p1 , imax )
      if ( ~status ) then
        result = [p1 p2]
        return
      end
      // Be p1 prime or composite, add its factors.
      result = [ result otherfactors1 ]
      // Factorize p2
      [ otherfactors2 , status ] = factor_fermatdo ( p2 , imax )
      if ( ~status ) then
        result = [p1 p2]
        return
      end
      // Be p2 prime or composite, add its factors.
      result = [ result otherfactors2 ]
    end
    status = %t
endfunction
//
// factor_fermatcore --
//   Given an odd integer, returns two 
//   factors p1 and p2 so that n = p1 x p2.
//   The factors may be non prime.
//   This is Algorithm C, "Factoring by addition and substraction", 
//   section 4.5.4 in Knuth's TAO
//   Returns p1 = 0 and p2 = 0 if the procedure fails 
//   before imax iterations.
// Arguments:
//   n : the positive, odd, integer number to factor 
//   imax : the maximum number of iterations in the search for a divisor
// Note :
// * The algorithm is very slow to factor 33554431.
//   factor_fermatdo ( 33554431 )
// * factor_fermatcore is faster than factor_fermatcore2
//
function [ p1 , p2 , status ] = factor_fermatcore ( n , imax )
    a = ceil(sqrt(n))
    status = %f
    for i = 1 : imax
      number_log ( sprintf ( "Loop #%d/%d, a = %d.\n" , i , imax , a ) )
      number_checkrange ( a * a )
      b2 = a * a - n
      // Computes if b is a square
      b = ceil(sqrt(b2))
      //mprintf ( "a=%d, b=%d, n=%d, a^2 - b^2=%d, |n-(a^2-b^2)|=%d\n", a, b, n, a^2 - b^2 , abs(n-a^2+b^2))
      if ( b^2 == b2 ) then
        // b2 is a square so that b2 = b^2 and 
        // n = a^2 - b^2
        number_log ( sprintf ( "Found b=%d, break.\n" , b ) )
        status = %t
        break
      end
      a = a + 1
    end
  // n = (a - b) * (a + b)
  if ( status ) then
    p1 = a - b
    p2 = a + b
    number_log ( sprintf ( "Found divisors p1=%d, p2=%d, done.\n" , p1 , p2 ) )
  else
    number_log ( sprintf ( "Found no divisor for n=%d.\n" , n ) )
    p1 = 0
    p2 = 0
  end
endfunction
//
// factor_fermatcore2 --
//   Given an odd integer, returns two 
//   factors p1 and p2 so that n = p1 x p2.
//   The factors may be non prime.
//   This is Algorithm C, "Factoring by addition and substraction", 
//   section 4.5.4 in Knuth's TAO
//   This is an implementation as close as possible to Knuth's
//   algorithm
//   Returns p1 = 0 and p2 = 0 if the procedure fails 
//   before imax iterations.
// Arguments:
//   n : the positive, odd, integer number to factor 
//   imax : the maximum number of iterations in the search for a divisor
// Note :
// The algorithm is very slow to factor 33554431.
// factor_fermatdo ( 33554431 )
//
function [ p1 , p2 , status ] = factor_fermatcore2 ( n , imax )
    step = 1
    status = %f
    for i = 1 : imax
      select step
      case 1
        // Initialize
        xp = 2 * ceil(sqrt(n)) + 1
        yp = 1
        r = (ceil(sqrt(n)))^2 - n
        step = 2
      case 2
        // Test r
        if ( r <= 0 ) then
          step = 4
        else
          step = 3
        end
      case 3
        // Step y
        r = r - yp
        yp = yp + 2
        step = 2
      case 4
        // Done ?
        if ( r == 0 ) then
          status = %t
          break
        else
          step = 5
        end
      case 5
        // Step x
        r = r + xp
        xp = xp + 2
        step = 3
      end
    end
  if ( status ) then
    p1 = ( xp - yp ) / 2
    p2 = ( xp + yp - 2 ) / 2
  else
    p1 = 0
    p2 = 0
  end
endfunction
//
// factor_pollard --
//   Returns the list of prime factors of n
//   Uses the Pollard method.
//   Returns 0 if the method fails to compute the factors.
// Arguments:
//   n : the positive integer number to factor
//   imax : maximum number of loops in Floyd's loop.
//
function [ result , status ] = factor_pollard ( n , imax )
    result = []
    // Factors by two until n is odd
    [ twofactors , status ] = factor_bytwo ( n , imax )
    if ( ~status ) then
      result = []
      return
    end
    twolist = twofactors ( 1 : $-1 )
    result  = [ result twolist ]
    u = twofactors ( $ )
    // Compute factors for odd number u
    if ( u <> 1 ) then
      [ otherfactors , status ] = pollardrhoglobal ( u , imax )
      result = [ result otherfactors ]
      result = gsort ( result , "g" , "i" )
    end
endfunction
//
// pollardrhoglobal --
//   Uses the Pollard - rho method to find new factors
//   until all factors have been found.
//   Returns status = %f if the method fails to compute the factors.
// Arguments:
//   n : the positive integer number to factor
//   imax : maximum number of loops in Floyd's loop.
// Bibliography
//   "Introduction to algorithms", Cormen, Leiserson, Rivest, Stein, 2nd edition
//
function [ result , status ] = pollardrhoglobal ( n , imax )
    number_log ( sprintf ( "pollardrhoglobal: n=%d\n" , n ) )
    u = n
    result = []
    simax = floor ( sqrt(imax) )
    for j = 1 : simax
      number_log ( sprintf ( "Global loop #%d/%d, u = %s\n" , j , simax , string ( u ) ) )
      // Perform a probable primality test to see if we continue.
      isprime = number_probableprime ( u )
      if ( isprime ) then
        number_log ( sprintf ( "Number u = %s is prime: we stop.\n" , string(u) ) )
        status = %t
        result ( 1 , $ + 1 ) = u
        return
      end
      [ d , status ] = factor_pollardrho ( u , simax )
      if ( status ) then
        // Check that the divisor is itself a prime
        isprime = number_probableprime ( d )
        if ( isprime ) then
          number_log ( sprintf ( "Divisor %s is prime, add it to the list.\n" , string ( d ) ) )
          result ( 1 , $ + 1 ) = d
          u = u / d
        else
          number_log ( sprintf ( "Divisor %s is NOT prime, factor it.\n" , string ( d ) ) )
          [ dresult , status ] = pollardrhoglobal ( d , imax )
          if ( status ) then
            number_log ( sprintf ( "Factoring of divisor OK : add [%s] to the list .\n" , strcat ( string ( dresult ) , " " ) ) )
            result = [ result dresult ]
            u = u / prod ( dresult )
          else
            number_log ( sprintf ( "Failure of Pollard-rho\n" ) )
            result ( 1 , $ + 1 ) = u
            return
          end
        end
      else
        number_log ( sprintf ( "Failure of Pollard-rho\n" ) )
        result ( 1 , $ + 1 ) = u
        return
      end
    end
    status = %f
    result ( 1 , $ + 1 ) = u
endfunction
//
// factor_pollardrho --
//   Returns the list of prime factors of the odd number n
//   Uses the Pollard - rho method.
//   Returns status = %f if the method fails to compute the factors.
// Arguments:
//   n : the positive integer number to factor
//   imax : maximum number of loops in Floyd's loop.
// Bibliography
//   "Introduction to algorithms", Cormen, Leiserson, Rivest, Stein, 2nd edition
//
function [ d , status ] = factor_pollardrho ( n , imax )
    number_log ( sprintf ( "factor_pollardrho: n=%d\n" , n ) )
    if ( n == 1 ) then
      result = 1
      status = %t
      return
    end
    // Try several values of c before declaring we are done
    seeds = number_cget ( "-pollardseeds" )
    for c = seeds
      i = 1
      xi = randominteger ( 1 , 1 , 0 , n - 1 )
      number_log ( sprintf ( "x=%d\n" , xi ) )
      y = xi
      k = 2
      for j = 1 : imax
        number_log ( sprintf ( "Local loop #%d/%d\n" , j , imax ) )
        i = i + 1
        xn = pmodulo ( xi^2 - c , n )
        number_log ( sprintf ( "x=%d\n" , xn ) )
        d = number_gcd ( y - xn , n )
        if ( ( d <> 1 ) & ( d <> n ) ) then
          number_log ( sprintf ( "Found new factor: d=%d\n" , d ) )
          status = %t
          return
        end
        if ( i == k ) then
          y = xn
          k = 2 * k
        end
        xi = xn
      end
    end
    status = %f
    d = []
endfunction
//
// randominteger --
//   Generates a matrix of random integers in [a , b]
//
function r = randominteger ( nbrows , nbcols , a , b )
  r = floor ( rand ( nbrows , nbcols ) * ( b - a + 1 ) ) + a 
endfunction
//
// factor_bytwo --
//   Divide the given n by 2 until the number becomes odd.
//   Returns the list of 2 which were found plus the odd
//   number at the end of the list.
// Arguments:
//   n : the positive integer number to factor
//
function [ result , status ] = factor_bytwo ( n , imax )
    result = []
    // Divide by 2 until the remainder is not zero
    u = n
    status = %f
    for i = 1 : imax
      number_log ( sprintf ( "Loop #%d/%d, u=%d\n" , i , imax , u ) )
      if ( modulo ( u , 2 ) == 0 ) then
        number_log ( sprintf ( "Found divisor %d\n" , 2 ) )
        result ( 1 , $ + 1 ) = 2
        u  = u / 2
      else 
        // Number is odd
        number_log ( sprintf ( "No divisor left, done.\n" ) )
        status = %t
        break
      end
    end
    result ( 1 , $ + 1 ) = u
endfunction
//
// factorization_memorylesstd --
//   Uses the Trial Division method, without any list of primes.
// Arguments:
//   n : the positive integer number to factor
//   imax : the maximum number of iterations in the search for a divisor
// Note:
//   This is Algorithm A, "Factoring by division", section 4.5.4 in Knuth's TAO
// Divide by prime until the remainder is not zero
// * add2 is a boolean which allows to compute the next prime
//   The following primes are tested in order:
//   2 3 5 7 11 13 17 ...
//   After the prime p=5 has been used, we add 2, then 4, then 2.
// * passedS is true when we have tried a prime greater or equal to sqrt(n)
//
function [ result , status ] = factor_memorylesstd ( n , imax )
    result = []
    // Get all primes up to sqrt(n)
    s = ceil ( sqrt ( n ) )
    u = n
    p = 2
    passedS = %f
    add2 = %t
    status = %f
    for i = 1 : imax
      number_log ( sprintf ( "Loop #%d/%d, u=%s\n" , i , imax , string ( u ) ) )
      if ( u == 1 ) then
        number_log ( sprintf ( "Found u=1, done.\n" ) )
        status = %t
        break
      end
      r = modulo ( u , p )
      if ( r==0 ) then
        number_log ( sprintf ( "Found divisor %s\n" , string( p )) )
        result ( 1 , $ + 1 ) = p
        u = u / p
      else
        // p does not divide u
        if ( ~passedS ) then
          // There are primes left to test
          if ( p == 2 ) then
            p = 3
          elseif ( p == 3 ) then
            p = 5
          elseif ( add2 ) then
            p = p + 2
            add2 = %f
          else
            p = p + 4
            add2 = %t
          end
          if ( p >= s ) then
            passedS = %t
          end
          number_log ( sprintf ( "Next divisor: %s\n" , string ( p ) ) )
        else
          // There are no primes left to test : the last number n is a prime
          number_log ( sprintf ( "No prime left to test, done.\n" ) )
          result ( 1 , $ + 1 ) = u
          status = %t
          break
        end
      end
    end
    if ( ~status ) then
      number_log ( sprintf ( "Failed to factor completely, u = %s.\n" , string ( u ) ) )
      result ( 1 , $ + 1 ) = u
    end
endfunction

