// Copyright (C) 2010 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function n = number_frombary ( varargin )
  // Compute a number from a matrix of digits.
  //
  // Calling Sequence
  //   n = number_frombary ( d )
  //   n = number_frombary ( d , basis )
  //   n = number_frombary ( d , basis , order )
  //
  // Parameters
  //   basis : a 1x1 matrix of floating point integers, the basis (default 2)
  //   d : a matrix of floating point integers, the digits
  //   order : a 1x1 matrix of strings, the order (default "littleendian")
  //   n : a 1x1 matrix of doubles
  //
  // Description
  //   Given a matrix of digits d(i), i=1,2,...,p,
  //   and a basis b, returns the "littleendian" number
  //   <programlisting>
  //   n = d(p) + d(p-1)*b + d(p-2)*b^2 + ... + d(1)*b^(p-1)
  //   </programlisting>
  //   If "bigendian" order is chosen, returns the number
  //   <programlisting>
  //   n = d(1) + d(2)*b + d(3)*b^2 + ... + d(p)*b^(p-1)
  //   </programlisting>
  //
  // Examples
  // n = number_frombary ( [1 0 0] ) // 4
  // basis = 2;
  // n = number_frombary ( [1 0 0] , basis ) // 4
  // n = number_frombary ( [1 0 0] , basis , "littleendian" ) // 4
  // n = number_frombary ( [0 0 1] , basis , "bigendian" ) // 4
  //
  // Authors
  // Copyright (C) 2010 - Michael Baudin
  //

  [lhs,rhs]=argn();
  if ( ( rhs <> 1 ) & ( rhs <> 2 ) & ( rhs <> 3 ) ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while %d to %d are expected."), "number_frombary" , rhs , 1 , 3 );
    error(errmsg)
  end
  d = varargin ( 1 )
  
  if ( rhs < 2 ) then
    basis = 2
  else
    basis = varargin(2)
  end
  
  if ( rhs < 3 ) then
    order = "littleendian"
  else
    order = varargin(3)
  end

  if ( basis < 2 ) then
    errmsg = msprintf(gettext("%s: Unexpected value for basis: Expected an integer greater than 2 but got %d instead."), "number_frombary" , basis );
    error(errmsg)
  end
  
  select order
  case "littleendian"
    n = 0
    p = size(d,"*")
    q = basis^(p-1)
    for i = 1 : p
      n = n + d(i) * q
      q = q / basis
    end
  case "bigendian"
    n = 0
    q = 1
    p = size(d,"*")
    for i = 1 : p
      n = n + d(i) * q
      q = q * basis
    end
  else
    error ( mprintf ( gettext ( "%s: Unknown order"  ), "number_tobary" ) )
  end
endfunction


