// Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin

// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function isprime = number_probableprime ( varargin )
  // Check if a number is a probable prime.
  //
  // Calling Sequence
  //   isprime = number_probableprime ( n )
  //   isprime = number_probableprime ( n , s )
  //
  // Parameters
  //   n : a 1x1 matrix of floating point integers, must be positive, the positive integer to test for primality
  //   s : a 1x1 matrix of floating point integers, the number of loops in the test for primality (default 50)
  //   isprime : a 1x1 matrix of booleans
  //
  // Description
  //   Returns %f if the number is a composite.
  //   Returns %t if the number is a probable prime.
  //   Uses Miller-Rabin randomized primality test.
  //
  // Examples
  // // Check for actual primes
  // number_probableprime ( 7 ) // %t
  // number_probableprime ( 5 ) // %t
  // number_probableprime ( 5 , 3 ) // %t
  // // Check for composite numbers
  // number_probableprime ( 10 ) // %f
  // number_probableprime ( 20 ) // %f
  // // Check for pseudo-primes : while pseudo prime does not detect that these numbers
  // // are composite, probable prime does it perfectly.
  // number_probableprime ( 341 ) // %f
  // number_probableprime ( 561 ) // %f
  // number_probableprime ( 645 ) // %f
  // number_probableprime ( 1105 ) // %f
  //
  // Bibliography
  //   "Introduction to algorithms", Cormen, Leiserson, Rivest, Stein, 2nd edition
  //
  // Authors
  // Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
  //

  [lhs,rhs]=argn();
  if ( ( rhs <> 1 ) & ( rhs <> 2 ) ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while %d or %d are expected."), "number_probableprime" , rhs , 1 , 2 );
    error(errmsg)
  end
  select rhs
  case 1 then
    n = varargin ( 1 )
    s = 50
  case 2 then
    n = varargin ( 1 )
    s = varargin ( 2 )
  end
  number_check ( n )
  number_check ( s )
  number_checkpositive ( "n" , n )
  number_checkpositive ( "s" , s )
  if ( or ( n == [0 1] ) ) then
    isprime = %f
    return
  end
  for j = 1 : s
    // Generates an random integer in [1 , n-1]
    a = randominteger ( 1 , 1 , 1 , n - 1 )
    if ( witness ( a , n ) ) then 
      number_log ( sprintf ( "Found witness a = %s\n" , string ( a ) ) )
      isprime = %f
      // Definitely
      return
    end
  end
  isprime = %t
  // Almost surely
endfunction
//
// randominteger --
//   Generates a matrix of random integers in [a , b]
//
function r = randominteger ( nbrows , nbcols , a , b )
  r = floor ( rand ( nbrows , nbcols ) * ( b - a + 1 ) ) + a 
endfunction
//
// witness --
//   Returns %t if a is a witness of the compositeness 
//   of n, that is, if it is possible to prove that n is composite.
//
function iscomposite = witness ( a , n )
  // Let n-1 = u 2^t where u is odd
  [ t , u ] = factorby2 ( n - 1 )
  xi = number_powermod ( a , u , n )
  for i = 1 : t
    xnext = number_powermod ( xi , 2 , n )
    if ( ( xnext == 1 ) & ( xi <> 1 ) & ( xi <> n-1 ) ) then
      iscomposite = %t
      return
    end
    xi = xnext
  end
  if ( xi <> 1 ) then
    iscomposite = %t
  else
    iscomposite = %f
  end
endfunction
//
// factorby2 --
//   Searches for u and t so that n = u 2^t
// Examples
// [ t , u ] = factorby2 ( 12 ), 2^t * u
// [ t , u ] = factorby2 ( 13 ), 2^t * u
// [ t , u ] = factorby2 ( 11 ), 2^t * u
// [ t , u ] = factorby2 ( 10 ), 2^t * u
// [ t , u ] = factorby2 ( 8 ), 2^t * u
//
function [ t , u ] = factorby2 ( n )
  t = 0
  u = n
  while ( %t )
    if ( modulo ( u , 2 ) == 1 ) then
      break
    end
    u = u / 2
    t = t + 1
  end
endfunction


