// Copyright (C) 2008-2009 - INRIA - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function digits = number_tobary ( varargin )
  //  Decompose a number into arbitrary basis.
  //
  // Calling Sequence
  //   digits = number_tobary ( n )
  //   digits = number_tobary ( n , basis )
  //   digits = number_tobary ( n , basis , order )
  //   digits = number_tobary ( n , basis , order , p )
  //
  // Parameters
  //   n : the integer to decompose, must be positive
  //   basis : the basis (default 2), must be positive
  //   order : a 1x1 matrix of strings, the order (default "littleendian")
  //   p : the number of digits on output.
  //
  // Description
  //   Returns a column matrix of digits of the decomposition of
  //   n in base b, i.e. decompose n as
  //   <programlisting>
  //   n = d(1) b^(p-1) + d(2) b^(p-2) + ... + d(p) b^0.
  //   </programlisting>
  //   This order is little endian order since the last digit
  //   is associated with b^0.
  //   If "bigendian" order is chosen, returns the digits d so that
  //   <programlisting>
  //   n = d(1) b^0 + d(2) b^1 + ... + d(p) b^(p-1),
  //   </programlisting>
  //   i.e. the last digit is associated with b^(p-1).
  //   If p is a positive integer, returns p digits.
  //   If the integer cannot be stored in p digits, generates an error.
  //   If the integer is larger than p digits, pad with zeros.
  //
  // Examples
  //   number_tobary (4,2) // [1 0 0]
  //   number_tobary (4,2,"bigendian") // [0 0 1]
  //   number_tobary (4,2,"littleendian",8) // [0,0,0,0,0,1,0,0]
  //
  // Bibliography
  //   Monte-Carlo methods in Financial Engineering, Paul Glasserman
  //
  // Authors
  // Copyright (C) 2008-2009 - INRIA - Michael Baudin
  // Copyright (C) 2010 - DIGITEO - Michael Baudin
  //

  [lhs,rhs]=argn();
  if ( ( rhs < 1 ) | ( rhs > 4 ) ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while %d to %d are expected."), "number_tobary" , rhs , 1 , 4 );
    error(errmsg)
  end
  //
  n = varargin ( 1 )
  if ( rhs < 2 ) then
    basis = 2
  else
    basis = varargin ( 2 )
  end
  if ( rhs < 3 ) then
    order = "littleendian";
  else
    order = varargin ( 3 )
  end
  if ( rhs < 4 ) then
    p = 0
  else
    p = varargin ( 4 )
  end
  //
  number_check ( n )
  number_check ( basis )
  number_checkpositive ( "n" , n )
  number_checkpositive ( "basis" , basis )
  
  if ( basis < 2 ) then
    errmsg = msprintf(gettext("%s: Unexpected value for basis: Expected an integer greater than 2 but got %d instead."), "number_bary" , basis );
    error(errmsg)
  end
  
  // Compute in littleendian
  if (n==0) then
    digits = 0;
  else
    jmax = int(log(n)/log(basis));
    q = int(basis^jmax);
    for j=1:jmax+1
      aj = int(n/q);
      digits(j) = aj;
      n = n - q * aj;
      q = q/basis;
    end
  end
  // Reverse order if necessary
  if ( order == "bigendian" ) then
    q = size(digits,"*")
    digits(q:-1:1) = digits(1:q)
  end
  // Pad up to p digits
  if ( p > 0 ) then
    q = size(digits,"*")
    if ( q > p ) then
      errmsg = msprintf(gettext("%s: Unexpected value %d to convert: %d bits are required, which is larger than maximum %d."), "number_tbgui_convertobin", n , q , p);
      error(errmsg)
    end
    select order
    case "littleendian"
      // Put zeros at beginning
      digits = [
      zeros(p-q,1)
      digits
      ];
    case "bigendian"
      // Put zeros at end
      digits = [
      digits
      zeros(p-q,1)
      ];
    else
      errmsg = msprintf(gettext("%s: Unexpected order: %d."), "number_tbgui_littlebig", number_baryguih.hlittlebigendian.Value);
      error(errmsg)
    end
  end
endfunction


