// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

// Reference
// Factorizations of bn±1,
// b = 2, 3, 5, 6, 7,10, 11, 12
// Up to High Powers
// Third Edition
// John Brillhart, D. H. Lehmer
// J. L. Selfridge, Bryant Tuckerman,
// and S. S. Wagstaff, Jr.

function assertfactor ( n , expected )
  mprintf ( "Factoring n=%d\n", n )
  computed = number_factor ( n , method );
  assert_equal ( prod(computed) , n );
  mprintf ( "> Factors f=[%s]\n", strcat ( string ( computed ) , " " ) )
  assert_equal ( computed , expected );
endfunction

function testfactor ( method )
  assertfactor ( 1 , 1 )
  for n = 2 : 30
    // Exceptions
    if ( method == "pollard-floyd" ) then
      if ( ( n==9 ) | ( n==18 ) | ( n==27 ) ) then
        // Skip this test : known case of failure
        continue
      end
    end
    mprintf ( "===================\n" )
    mprintf ( "Factoring n=%d\n", n )
    computed = number_factor ( n , method );
    mprintf ( "> Factors f=[%s]\n", strcat ( string ( computed ) , " " ) )
    assert_equal ( prod(computed) , n );
    for p = computed
      isprime = number_isprime ( p )
      assert_equal ( isprime , %t );
    end
  end
  assertfactor ( 2^2-1 , 3 )
  assertfactor ( 2^3-1 , 7 )
  assertfactor ( 2^4-1 , [3 5] )
  assertfactor ( 2^5-1 , 31 )
  if ( method <> "pollard-floyd" ) then
    assertfactor ( 2^6-1 , [3 3 7] )
  end
  assertfactor ( 2^7-1 , 127 )
  assertfactor ( 2^8-1 , [3 5 17] )
  assertfactor ( 2^9-1 , [7 73] )
  assertfactor ( 2^11-1 , [23 89] )
  if ( method <> "pollard-floyd" ) then
    assertfactor ( 2^12-1 , [3 3 5 7 13] )
  end
  assertfactor ( 2^13-1 , 8191 )
  assertfactor ( 2^14-1 , [3 43 127] )
  if ( method <> "pollard-floyd" ) then
    assertfactor ( 2^15-1 , [7 31 151] )
  end
endfunction

testfactor ( "fasttrialdivision" )
testfactor ( "trialdivision" )
testfactor ( "memorylesstd" )
testfactor ( "fermat" )
rand ( "seed" , 0 )
testfactor ( "pollard-rho" )

// Check that verbose mode is fonctionnal
number_configure ( "-verbose" , 1 );
number_factor ( 10 , "fasttrialdivision" );
number_factor ( 10 , "trialdivision" );
number_factor ( 10 , "memorylesstd" );
number_factor ( 10 , "fermat" );
number_factor ( 10 , "pollard-rho" );
number_configure ( "-verbose" , 0 );


